
module FileOperations

  def mkdir_p( dname, prefix = nil )
    dname = prefix + dname if prefix
    $stderr.puts "mkdir -p #{dname}" if verbose?
    return if no_harm?

    # does not check '/'... it's too abnormal case
    dirs = dname.split(%r<(?=/)>)
    if /\A[a-z]:\z/i === dirs[0]
      disk = dirs.shift
      dirs[0] = disk + dirs[0]
    end
    dirs.each_index do |idx|
      path = dirs[0..idx].join('')
      Dir.mkdir path unless dir? path
    end
  end

  def rm_f( fname )
    $stderr.puts "rm -f #{fname}" if verbose?
    return if no_harm?

    if File.exist? fname or File.symlink? fname
      File.chmod 0777, fname
      File.unlink fname
    end
  end

  def rm_rf( dn )
    $stderr.puts "rm -rf #{dn}" if verbose?
    return if no_harm?

    Dir.chdir dn
    Dir.foreach('.') do |fn|
      next if fn == '.'
      next if fn == '..'
      if dir? fn
        verbose_off {
            rm_rf fn
        }
      else
        verbose_off {
            rm_f fn
        }
      end
    end
    Dir.chdir '..'
    Dir.rmdir dn
  end

  def mv( src, dest )
    rm_f dest
    begin
      File.link src, dest
    rescue
      File.write dest, File.read_all(src)
      File.chmod File.stat(src).mode, dest
    end
    rm_f src
  end

  def install( from, dest, mode, prefix = nil )
    $stderr.puts "install #{from} #{dest}" if verbose?
    return if no_harm?

    realdest = prefix + dest if prefix
    if dir? realdest
      realdest += '/' + File.basename(from)
    end
    str = File.read_all(from)
    if diff? str, realdest
      verbose_off {
          rm_f realdest if File.exist? realdest
      }
      File.write realdest, str
      File.chmod mode, realdest

      File.open(objdir + '/InstalledFiles', 'a') {|f| f.puts realdest }
    end
  end

  def diff?( orig, targ )
    return true unless File.exist? targ
    orig != File.read_all(targ)
  end

  def command( str )
    $stderr.puts str if verbose?
    system str or raise RuntimeError, "'system #{str}' failed"
  end

  def ruby( str )
    command config('ruby-prog') + ' ' + str
  end

  def dir?( dname )
    # for corrupted windows stat()
    File.directory?((dname[-1,1] == '/') ? dname : dname + '/')
  end

  def all_files_in( dname )
    Dir.open(dname) {|d|
        return d.select {|n| FileTest.file? "#{dname}/#{n}" }
    }
  end

  REJECT_DIRS = %w(
    CVS SCCS RCS CVS.adm
  )

  def all_dirs_in( dname )
    Dir.open(dname) {|d|
        return d.select {|n| dir? "#{dname}/#{n}" } - %w(. ..) - REJECT_DIRS
    }
  end

end

